@@ -1,6 +1,6 @@
 /*
 
-  silcauth.c
+  silcauth.c 
 
   Author: Pekka Riikonen <priikone@silcnet.org>
 
@@ -8,8 +8,7 @@
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
-  the Free Software Foundation; either version 2 of the License, or
-  (at your option) any later version.
+  the Free Software Foundation; version 2 of the License.
 
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -17,7 +16,7 @@
   GNU General Public License for more details.
 
 */
-/* $Id: silcauth.c,v 1.18.2.2 2002/04/15 19:46:20 johnny Exp $ */
+/* $Id: silcauth.c,v 1.28.2.2 2003/01/14 11:43:38 priikone Exp $ */
 
 #include "silcincludes.h"
 #include "silcauth.h"
@@ -68,7 +67,8 @@ SilcAuthPayload silc_auth_payload_parse(const unsigned char *data,
     return NULL;
   }
 
-  if (newp->len != buffer.len) {
+  if (newp->len != buffer.len || 
+      newp->random_len + newp->auth_len > buffer.len - 8) {
     silc_auth_payload_free(newp);
     return NULL;
   }
@@ -104,13 +104,28 @@ SilcBuffer silc_auth_payload_encode(SilcAuthMethod method,
 {
   SilcBuffer buffer;
   SilcUInt32 len;
+  unsigned char *autf8 = NULL;
+  SilcUInt32 autf8_len;
 
   SILC_LOG_DEBUG(("Encoding Authentication Payload"));
 
+  /* Passphrase MUST be UTF-8 encoded, encode if it is not */
+  if (method == SILC_AUTH_PASSWORD && !silc_utf8_valid(auth_data, auth_len)) {
+    autf8_len = silc_utf8_encoded_len(auth_data, auth_len, 0);
+    if (!autf8_len)
+      return NULL;
+    autf8 = silc_calloc(autf8_len, sizeof(*autf8));
+    auth_len = silc_utf8_encode(auth_data, auth_len, 0, autf8, autf8_len);
+    auth_data = (const unsigned char *)autf8;
+  }
+
   len = 2 + 2 + 2 + random_len + 2 + auth_len;
   buffer = silc_buffer_alloc_size(len);
-  if (!buffer)
+  if (!buffer) {
+    silc_free(autf8);
     return NULL;
+  }
+
   silc_buffer_format(buffer,
 		     SILC_STR_UI_SHORT(len),
 		     SILC_STR_UI_SHORT(method),
@@ -120,6 +135,7 @@ SilcBuffer silc_auth_payload_encode(SilcAuthMethod method,
 		     SILC_STR_UI_XNSTRING(auth_data, auth_len),
 		     SILC_STR_END);
 
+  silc_free(autf8);
   return buffer;
 }
 
@@ -147,7 +163,7 @@ SilcAuthMethod silc_auth_get_method(SilcAuthPayload payload)
   return payload->auth_method;
 }
 
-/* Get the authentication data */
+/* Get the authentication data. If this is passphrase it is UTF-8 encoded. */
 
 unsigned char *silc_auth_get_data(SilcAuthPayload payload,
 				  SilcUInt32 *auth_len)
@@ -169,7 +185,7 @@ unsigned char *silc_auth_get_data(SilcAuthPayload payload,
 
 static unsigned char *
 silc_auth_public_key_encode_data(SilcPublicKey public_key,
-				 const unsigned char *random,
+				 const unsigned char *randomdata,
 				 SilcUInt32 random_len, const void *id,
 				 SilcIdType type, SilcUInt32 *ret_len)
 {
@@ -195,17 +211,12 @@ silc_auth_public_key_encode_data(SilcPublicKey public_key,
     return NULL;
   }
   silc_buffer_format(buf,
-		     SILC_STR_UI_XNSTRING(random, random_len),
+		     SILC_STR_UI_XNSTRING(randomdata, random_len),
 		     SILC_STR_UI_XNSTRING(id_data, id_len),
 		     SILC_STR_UI_XNSTRING(pk, pk_len),
 		     SILC_STR_END);
 
-  ret = silc_memdup(buf->data, buf->len);
-  if (!ret)
-    return NULL;
-
-  if (ret_len)
-    *ret_len = buf->len;
+  ret = silc_buffer_steal(buf, ret_len);
 
   silc_buffer_free(buf);
   silc_free(id_data);
@@ -223,8 +234,8 @@ SilcBuffer silc_auth_public_key_auth_generate(SilcPublicKey public_key,
 					      SilcRng rng, SilcHash hash,
 					      const void *id, SilcIdType type)
 {
-  unsigned char *random;
-  unsigned char auth_data[1024];
+  unsigned char *randomdata;
+  unsigned char auth_data[2048 + 1];
   SilcUInt32 auth_len;
   unsigned char *tmp;
   SilcUInt32 tmp_len;
@@ -235,20 +246,20 @@ SilcBuffer silc_auth_public_key_auth_generate(SilcPublicKey public_key,
 
   /* Get 256 bytes of random data */
   if (rng)
-    random = silc_rng_get_rn_data(rng, 256);
+    randomdata = silc_rng_get_rn_data(rng, 256);
   else
-    random = silc_rng_global_get_rn_data(256);
-  if (!random)
+    randomdata = silc_rng_global_get_rn_data(256);
+  if (!randomdata)
     return NULL;
 
   /* Encode the auth data */
-  tmp = silc_auth_public_key_encode_data(public_key, random, 256, id, type,
-					 &tmp_len);
+  tmp = silc_auth_public_key_encode_data(public_key, randomdata, 256, id, 
+					 type, &tmp_len);
   if (!tmp)
     return NULL;
 
   /* Allocate PKCS object */
-  if (!silc_pkcs_alloc(public_key->name, &pkcs)) {
+  if (!silc_pkcs_alloc(private_key->name, &pkcs)) {
     memset(tmp, 0, tmp_len);
     silc_free(tmp);
     return NULL;
@@ -257,25 +268,26 @@ SilcBuffer silc_auth_public_key_auth_generate(SilcPublicKey public_key,
   silc_pkcs_private_key_set(pkcs, private_key);
 
   /* Compute the hash and the signature. */
-  if (!silc_pkcs_sign_with_hash(pkcs, hash, tmp, tmp_len, auth_data,
+  if (silc_pkcs_get_key_len(pkcs) / 8 > sizeof(auth_data) - 1 ||
+      !silc_pkcs_sign_with_hash(pkcs, hash, tmp, tmp_len, auth_data,
 				&auth_len)) {
-    memset(random, 0, 256);
+    memset(randomdata, 0, 256);
     memset(tmp, 0, tmp_len);
     silc_free(tmp);
-    silc_free(random);
+    silc_free(randomdata);
     silc_pkcs_free(pkcs);
     return NULL;
   }
 
   /* Encode Authentication Payload */
-  buf = silc_auth_payload_encode(SILC_AUTH_PUBLIC_KEY, random, 256,
+  buf = silc_auth_payload_encode(SILC_AUTH_PUBLIC_KEY, randomdata, 256,
 				 auth_data, auth_len);
 
   memset(tmp, 0, tmp_len);
   memset(auth_data, 0, sizeof(auth_data));
-  memset(random, 0, 256);
+  memset(randomdata, 0, 256);
   silc_free(tmp);
-  silc_free(random);
+  silc_free(randomdata);
   silc_pkcs_free(pkcs);
 
   return buf;
