@@ -16,7 +16,7 @@
   GNU General Public License for more details.
 
 */
-/* $Id: silchmac.c,v 1.22 2002/06/17 19:54:17 priikone Exp $ */
+/* $Id: silchmac.c,v 1.24 2002/11/16 11:16:06 priikone Exp $ */
 
 #include "silcincludes.h"
 
@@ -185,7 +185,7 @@ bool silc_hmac_unregister_all(void)
    of the HMAC.  If it is NULL then the hash function is allocated and
    the name of the hash algorithm is derived from the `name'. */
 
-bool silc_hmac_alloc(char *name, SilcHash hash, SilcHmac *new_hmac)
+bool silc_hmac_alloc(const char *name, SilcHash hash, SilcHmac *new_hmac)
 {
   SILC_LOG_DEBUG(("Allocating new HMAC"));
 
@@ -429,7 +429,7 @@ void silc_hmac_init_with_key(SilcHmac hmac, const unsigned char *key,
 			     SilcUInt32 key_len)
 {
   SilcHash hash = hmac->hash;
-  silc_hmac_init_internal(hmac, hmac->key, hmac->key_len);
+  silc_hmac_init_internal(hmac, (unsigned char *)key, key_len);
   silc_hash_init(hash);
   silc_hash_update(hash, hmac->inner_pad, silc_hash_block_len(hash));
 }
